# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
The purpose of this extension is to provide a configuration value that can be
used to disable intersphinx on the command-line without editing conf.py. To use,
you can build documentation with::

    sphinx-build ... -D disable_intersphinx=1

This is used e.g. by astropy-helpers when using the build_docs command.
"""

from __future__ import print_function

from distutils.version import LooseVersion

from sphinx import __version__

SPHINX_LT_16 = LooseVersion(__version__) < LooseVersion('1.6')
SPHINX_LT_18 = LooseVersion(__version__) < LooseVersion('1.8')


def disable_intersphinx(app, config=None):

    from sphinx.util.console import bold

    if SPHINX_LT_16:
        info = app.info
    else:
        from sphinx.util import logging
        info = logging.getLogger(__name__).info

    if app.config.disable_intersphinx:
        info(bold('disabling intersphinx...'))
        app.config.intersphinx_mapping.clear()


def setup(app):

    # Note that the config-inited setting was only added in Sphinx 1.8. For
    # earlier versions we use builder-inited but we need to be careful in what
    # order the extensions are declared so that this happens before intersphinx.
    if SPHINX_LT_18:
        app.connect('builder-inited', disable_intersphinx)
    else:
        app.connect('config-inited', disable_intersphinx)

    app.add_config_value('disable_intersphinx', 0, True)



# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
The purpose of this extension is to give a clear warning if sphinx is expecting
a static directory to be present but it isn't.
"""

from __future__ import print_function

import os
from distutils.version import LooseVersion

from sphinx import __version__

SPHINX_LT_16 = LooseVersion(__version__) < LooseVersion('1.6')
SPHINX_LT_18 = LooseVersion(__version__) < LooseVersion('1.8')


WARNING_TEMPLATE = """
Note: The static directory '{0}' was not found. This is often because it is
      empty and you are using git. If so, you don't need it, so make
      sure it isn't included in the html_static_path setting in your conf.py
      file, otherwise Sphinx may fail the build if you are turning warnings into
      errors.
"""


def static_warning(app, config=None):

    if SPHINX_LT_16:
        info = app.info
    else:
        from sphinx.util import logging
        info = logging.getLogger(__name__).info

    for directory in app.config.html_static_path:
        if not os.path.exists(directory):
            info(WARNING_TEMPLATE.format(directory))


def setup(app):
    app.connect('build-finished', static_warning)



#!/usr/bin/env python

# Licensed under a 3-clause BSD style license - see LICENSE.rst

from setuptools import setup, find_packages

with open('README.rst') as infile:
    long_description = infile.read()

from sphinx_astropy import __version__

setup(name='sphinx-astropy',
      version=__version__,
      description='Sphinx extensions and configuration specific to the Astropy project',
      long_description=long_description,
      author='The Astropy Developers',
      author_email='astropy.team@gmail.com',
      license='BSD',
      url='http://astropy.org',
      zip_safe=False,
      install_requires=['sphinx>=1.4', 'astropy-sphinx-theme', 'numpydoc',
                        'sphinx-automodapi', 'sphinx-gallery', 'pillow'],
      packages=find_packages(),
      package_data={'sphinx_astropy': ['local/*']},
      classifiers=['Intended Audience :: Developers',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2',
                   'Programming Language :: Python :: 3',
                   'Operating System :: OS Independent',
                   'License :: OSI Approved :: BSD License'])



from distutils.version import LooseVersion

from sphinx import __version__

SPHINX_LT_17 = LooseVersion(__version__) < LooseVersion('1.7')

if SPHINX_LT_17:

    from sphinx import build_main as build_main_original

    # As of Sphinx 1.7, the first argument is now no longer ignored, so we
    # construct a wrapper for older versions.

    def build_main(argv=None):
        argv.insert(0, 'sphinx-build')
        return build_main_original(argv=argv)

else:
    from sphinx.cmd.build import build_main

BASIC_CONF = """
from sphinx_astropy.conf import *
suppress_warnings = ['app.add_directive', 'app.add_node', 'app.add_role']
"""

BASIC_INDEX = """
Title
=====

Just a test
"""


def generate_files(tmpdir):

    with open(tmpdir.join('conf.py').strpath, 'w') as f:
        f.write(BASIC_CONF)

    with open(tmpdir.join('index.rst').strpath, 'w') as f:
        f.write(BASIC_INDEX)


def test_conf(tmpdir):

    # Just make sure the docs build with the default sphinx-astropy configuration

    generate_files(tmpdir)

    src_dir = tmpdir.strpath
    html_dir = tmpdir.mkdir('html').strpath

    status = build_main(argv=['-W', '-b', 'html', src_dir, html_dir])

    assert status == 0


def test_intersphinx_toggle(tmpdir, capsys):

    # Test the sphinx_astropy.ext.intersphinx_toggle extension

    generate_files(tmpdir)

    src_dir = tmpdir.strpath
    html_dir = tmpdir.mkdir('html').strpath

    status = build_main(argv=['-W', '-b', 'html', src_dir, html_dir, '-D', 'disable_intersphinx=1'])

    assert status == 0

    captured = capsys.readouterr()
    assert 'disabling intersphinx' in captured.out
    assert 'loading intersphinx' not in captured.out



# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""
This sphinx extension makes the issue numbers in the changelog into links to
GitHub issues.
"""

from __future__ import print_function

import re

from docutils.nodes import Text, reference

BLOCK_PATTERN = re.compile('\[#.+\]', flags=re.DOTALL)
ISSUE_PATTERN = re.compile('#[0-9]+')


def process_changelog_links(app, doctree, docname):
    for rex in app.changelog_links_rexes:
        if rex.match(docname):
            break
    else:
        # if the doc doesn't match any of the changelog regexes, don't process
        return

    app.info('[changelog_links] Adding changelog links to "{0}"'.format(docname))

    for item in doctree.traverse():

        if not isinstance(item, Text):
            continue

        # We build a new list of items to replace the current item. If
        # a link is found, we need to use a 'reference' item.
        children = []

        # First cycle through blocks of issues (delimited by []) then
        # iterate inside each one to find the individual issues.
        prev_block_end = 0
        for block in BLOCK_PATTERN.finditer(item):
            block_start, block_end = block.start(), block.end()
            children.append(Text(item[prev_block_end:block_start]))
            block = item[block_start:block_end]
            prev_end = 0
            for m in ISSUE_PATTERN.finditer(block):
                start, end = m.start(), m.end()
                children.append(Text(block[prev_end:start]))
                issue_number = block[start:end]
                refuri = app.config.github_issues_url + issue_number[1:]
                children.append(reference(text=issue_number,
                                          name=issue_number,
                                          refuri=refuri))
                prev_end = end

            prev_block_end = block_end

            # If no issues were found, this adds the whole item,
            # otherwise it adds the remaining text.
            children.append(Text(block[prev_end:block_end]))

        # If no blocks were found, this adds the whole item, otherwise
        # it adds the remaining text.
        children.append(Text(item[prev_block_end:]))

        # Replace item by the new list of items we have generated,
        # which may contain links.
        item.parent.replace(item, children)


def setup_patterns_rexes(app):
    app.changelog_links_rexes = [re.compile(pat) for pat in
                                 app.config.changelog_links_docpattern]


def setup(app):

    app.connect('doctree-resolved', process_changelog_links)
    app.connect('builder-inited', setup_patterns_rexes)
    app.add_config_value('github_issues_url', None, True)
    app.add_config_value('changelog_links_docpattern', ['.*changelog.*', 'whatsnew/.*'], True)

    return {'parallel_read_safe': True,
            'parallel_write_safe': True}
